#include <proto/exec.h>
#include <exec/resident.h>
#include <exec/initializers.h>
#include <exec/execbase.h>
#include <exec/libraries.h>
#include <proto/dos.h>
#include <dos/dos.h>
#include <proto/intuition.h>
#include <graphics/gfxbase.h>

#define ColorWheelBase	ColorWheelBase_intern

#include "colorwheel_intern.h"

/* Reserved library-function and dummy entrypoint */
LONG LIB_Reserved(void)
{
	return 0;
}

struct Library *LibInit(struct ColorWheelBase_intern *ColorWheelBase, BPTR SegList, struct ExecBase *sysbase);
struct Library *LIB_Open(void);
BPTR LIB_Close(void);
BPTR LIB_Expunge(void);

extern void ConvertHSBToRGB_GATE( void ), ConvertRGBToHSB_GATE( void );

ULONG LibVectors[] = {
	FUNCARRAY_32BIT_NATIVE,
	(ULONG) &LIB_Open,
	(ULONG) &LIB_Close,
	(ULONG) &LIB_Expunge,
	(ULONG) &LIB_Reserved,

	(ULONG) &ConvertHSBToRGB_GATE,
	(ULONG) &ConvertRGBToHSB_GATE,
	0xFFFFFFFF
};

struct LibInitStruct
{
	ULONG	LibSize;
	void	*FuncTable;
	void	*DataTable;
	void	(*InitFunc)(void);
};

struct LibInitStruct LibInitStruct = {
	sizeof(struct ColorWheelBase_intern),
	LibVectors,
	NULL,
	(void (*)(void)) &LibInit
};

static const char LibVersion[] = VERSION_STRING;

static const struct Resident RomTag = {
	RTC_MATCHWORD,
	&RomTag,
	&RomTag + 1,
	RTF_PPC | RTF_AUTOINIT | RTF_EXTENDED,
	VERSION_NUMBER,
	NT_LIBRARY,
	0,
	NAME_STRING,
	&LibVersion[6],
	&LibInitStruct,
	REVISION_NUMBER,
	NULL
};

ULONG __abox__ = 1;

static void LibCleanup(struct ColorWheelBase_intern *ColorWheelBase)
{
	if (GfxBase) CloseLibrary((struct Library *)GfxBase);
	if (IntuitionBase) CloseLibrary((struct Library *)IntuitionBase);
	if (UtilityBase) CloseLibrary(UtilityBase);
	if (CyberGfxBase) CloseLibrary(CyberGfxBase);
	if (LayersBase) CloseLibrary(LayersBase);
}

BPTR LibExpunge(struct ColorWheelBase_intern *ColorWheelBase)
{
	if ((ColorWheelBase->library.lib_OpenCnt == 0))
	{
		BPTR seglist;

		if (ColorWheelBase->classptr)
		{
			if (!FreeClass(ColorWheelBase->classptr))
			{
				AddClass(ColorWheelBase->classptr);
				return 0L;
			}
		}

		seglist = ColorWheelBase->seglist;
		Remove((struct Node *)ColorWheelBase);
		LibCleanup(ColorWheelBase);

		FreeMem((APTR)((ULONG)(ColorWheelBase) - (ULONG)(ColorWheelBase->library.lib_NegSize)),
				ColorWheelBase->library.lib_NegSize + ColorWheelBase->library.lib_PosSize);

		return seglist;
	} else ColorWheelBase->library.lib_Flags |= LIBF_DELEXP;

	return 0L;
}

struct Library *LibInit(struct ColorWheelBase_intern *ColorWheelBase, BPTR SegList, struct ExecBase *sysbase)
{
	ColorWheelBase->library.lib_Revision = REVISION_NUMBER;
	SysBase = sysbase;
	ColorWheelBase->seglist = SegList;

	if ((GfxBase = OpenLibrary("graphics.library", 39)) &&
		(IntuitionBase = OpenLibrary("intuition.library", 39)) &&
		(LayersBase = OpenLibrary("layers.library", 39)) &&
		(UtilityBase = OpenLibrary("utility.library", 39)))
	{
		CyberGfxBase = OpenLibrary("cybergraphics.library", 40);

		if ((ColorWheelBase->classptr = InitColorWheelClass(ColorWheelBase)))
		{
			return((struct Library *)ColorWheelBase);
		}
	}

	LibCleanup(ColorWheelBase);

	FreeMem((APTR)((ULONG)(ColorWheelBase) - (ULONG)(ColorWheelBase->library.lib_NegSize)),
			ColorWheelBase->library.lib_NegSize + ColorWheelBase->library.lib_PosSize);
	
	return NULL;
}

struct Library *LIB_Open(void)
{
	struct ColorWheelBase_intern *ColorWheelBase = (struct ColorWheelBase_intern *)REG_A6;

	ColorWheelBase->library.lib_Flags &= ~LIBF_DELEXP;
	ColorWheelBase->library.lib_OpenCnt++;

	return((struct Library *)ColorWheelBase);
}

BPTR LIB_Close(void)
{
	struct ColorWheelBase_intern *ColorWheelBase = (struct ColorWheelBase_intern *)REG_A6;

	if (ColorWheelBase->library.lib_OpenCnt > 0) ColorWheelBase->library.lib_OpenCnt--;

	if ((ColorWheelBase->library.lib_OpenCnt == 0) && (ColorWheelBase->library.lib_Flags & LIBF_DELEXP))
		return LibExpunge(ColorWheelBase);

	return 0L;
}

BPTR LIB_Expunge(void)
{
	struct ColorWheelBase_intern *ColorWheelBase = (struct ColorWheelBase_intern *)REG_A6;

	return LibExpunge(ColorWheelBase);
}

